﻿using BPASmartClient.Message;
using BPASmartClient.MessageCommunication;
using BPASmartClient.MessageCommunication.MsgControl;
using BPASmartClient.SerialPort;
using System;
using System.Runtime.InteropServices;
using System.Threading;

namespace BPASmartClient.SCChip
{
    /// <summary>
    /// 指令封装
    /// </summary>
    internal class CommandHandler
    {
        private SerialPortClient commProxy;
        private ICChipPackage package = new ICChipPackage();

        /// <summary>
        /// 初始化
        /// </summary>
        internal void Init(SerialPortClient commProxy)
        {
            this.commProxy = commProxy;
            Class_InnerMessageBus.GetInstance().ListenMessage(this,Class_MessageName.SCChip_TakeCup,"TakeCupHandler");
            Class_InnerMessageBus.GetInstance().ListenMessage(this,Class_MessageName.SCChip_MakeIceCream,"MakeIceCreamHandler");
            Class_InnerMessageBus.GetInstance().ListenMessage(this,Class_MessageName.SCChip_SESwitchCream,"SESwitchCreamHandler");
            Class_InnerMessageBus.GetInstance().ListenMessage(this,Class_MessageName.SCChip_RotorSwitch,"RotorSwitchHandler");

        }

        /// <summary>
        /// STM32F103RCT6单片机舵机打开或者关闭
        /// </summary>
        public void SESwitchCreamHandler(object sender,InnerMessageEventArgs e)
        {
            try
            {
                if (e.obj_MessageObj is TakeCupEvent)
                {
                    ChipStatus.GetInstance().ArticleDist = 0;
                    package.Cmd = (e.obj_MessageObj as SESwitchCreamEvent).IsOpen ? IC_CMD.OPEN_SE : IC_CMD.CLOSE_SE;
                    package.Value = (byte)(e.obj_MessageObj as SESwitchCreamEvent).SteeringEngine;
                    commProxy.SendData(StructureToByte(package));
                }
            }
            catch (Exception ex)
            {
                MessageLog.GetInstance.Show($"BPASmartClient.SCChip 中引发错误,SESwitchCreamHandler 类,描述：[{ex.Message}]");
            }
        }

        /// <summary>
        /// STM32F103RCT6单片机控制冰淇淋机器转
        /// </summary>
        public void RotorSwitchHandler(object sender,InnerMessageEventArgs e)
        {
            try
            {
                if (e.obj_MessageObj is RotorSwitchEvent)
                {
                    package.Cmd = IC_CMD.ROTOR;
                    package.Value = (e.obj_MessageObj as RotorSwitchEvent).TurnOn ? (byte)IC_ROTOR.OPEN_ROTOR : (byte)IC_ROTOR.CLOSE_ROTOR;
                    commProxy.SendData(StructureToByte(package));
                }
            }
            catch (Exception ex)
            {
                MessageLog.GetInstance.Show($"BPASmartClient.SCChip 中引发错误,RotorSwitchHandler 类,描述：[{ex.Message}]");
            }
        }

        /// <summary>
        /// STM32F103RCT6单片机舵机打料
        /// </summary>
        public void MakeIceCreamHandler(object sender,InnerMessageEventArgs e)
        {
            try
            {
                if (e.obj_MessageObj is MakeIceCreamEvent)
                {
                    switch ((e.obj_MessageObj as MakeIceCreamEvent).SteeringEngine)
                    {
                        case IC_SE.SE_1:
                            ChipStatus.GetInstance().CompletedOpen_SE_1 = false;
                            break;
                        case IC_SE.SE_2:
                            ChipStatus.GetInstance().CompletedOpen_SE_2 = false;
                            break;
                        case IC_SE.SE_3:
                            ChipStatus.GetInstance().CompletedOpen_SE_3 = false;
                            break;
                    }
                    package.Cmd = IC_CMD.OPEN_SE;
                    package.Value = (byte)(e.obj_MessageObj as MakeIceCreamEvent).SteeringEngine;
                    commProxy.SendData(StructureToByte(package));

                    bool wait = true;
                    DateTime waitTimeout = DateTime.Now.AddSeconds(3);
                    while (wait)
                    {
                        wait = DateTime.Now < waitTimeout;
                        if (wait)
                        {
                            switch ((e.obj_MessageObj as MakeIceCreamEvent).SteeringEngine)
                            {
                                case IC_SE.SE_1:
                                    wait = !ChipStatus.GetInstance().CompletedOpen_SE_1;
                                    break;
                                case IC_SE.SE_2:
                                    wait = !ChipStatus.GetInstance().CompletedOpen_SE_2;
                                    break;
                                case IC_SE.SE_3:
                                    wait = !ChipStatus.GetInstance().CompletedOpen_SE_3;
                                    break;
                            }
                        }
                        Thread.Sleep(10);
                    }
                    Thread.Sleep(2000);
                    package.Cmd = IC_CMD.CLOSE_SE;
                    package.Value = (byte)(e.obj_MessageObj as MakeIceCreamEvent).SteeringEngine;
                    commProxy.SendData(StructureToByte(package));

                    wait = true;
                    waitTimeout = DateTime.Now.AddSeconds(3);
                    while (wait)
                    {
                        wait = DateTime.Now < waitTimeout;
                        if (wait)
                        {
                            switch ((e.obj_MessageObj as MakeIceCreamEvent).SteeringEngine)
                            {
                                case IC_SE.SE_1:
                                    wait = !ChipStatus.GetInstance().CompletedClose_SE_1;
                                    break;
                                case IC_SE.SE_2:
                                    wait = !ChipStatus.GetInstance().CompletedClose_SE_2;
                                    break;
                                case IC_SE.SE_3:
                                    wait = !ChipStatus.GetInstance().CompletedClose_SE_3;
                                    break;
                            }
                        }
                        Thread.Sleep(10);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageLog.GetInstance.Show($"BPASmartClient.SCChip 中引发错误,MakeIceCreamHandler 类,描述：[{ex.Message}]");
            }
        }

        /// <summary>
        /// STM32F103RCT6单片机下杯
        /// </summary>
        public void TakeCupHandler(object sender,InnerMessageEventArgs e)
        {
            try
            {
                if (e.obj_MessageObj is TakeCupEvent)
                {
                    switch ((e.obj_MessageObj as TakeCupEvent).Cup)
                    {
                        case IC_CUP.CUP_ICECREAM:
                            ChipStatus.GetInstance().CompletedTake_CPU_CUP_ICECREAM = false;
                            break;
                        case IC_CUP.CUP_COFFEE:
                            ChipStatus.GetInstance().CompletedTake_CPU_CUP_COFFEE = false;
                            break;
                    }
                    package.Cmd = IC_CMD.TAKE_CUP;
                    package.Value = (byte)(e.obj_MessageObj as TakeCupEvent).Cup;
                    commProxy.SendData(StructureToByte(package));

                    bool wait = true;
                    var waitTimeout = DateTime.Now.AddSeconds(3);
                    while (wait)
                    {
                        wait = DateTime.Now < waitTimeout;
                        if (wait)
                        {
                            switch ((e.obj_MessageObj as TakeCupEvent).Cup)
                            {
                                case IC_CUP.CUP_ICECREAM:
                                    wait = !ChipStatus.GetInstance().CompletedTake_CPU_CUP_ICECREAM;
                                    break;
                                case IC_CUP.CUP_COFFEE:
                                    wait = !ChipStatus.GetInstance().CompletedTake_CPU_CUP_COFFEE;
                                    break;
                            }
                        }
                        Thread.Sleep(10);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageLog.GetInstance.Show($"BPASmartClient.SCChip 中引发错误,TakeCupHandler 类,描述：[{ex.Message}]");
            }
        }

        private byte[] StructureToByte(ICChipPackage structure)
        {
            structure.Header = 0xAA;
            structure.Sender = IC_SENDER.CONSOLE;
            structure.End = 0xBB;

            int size = Marshal.SizeOf(typeof(ICChipPackage));
            byte[] buffer = new byte[size];
            IntPtr bufferIntPtr = Marshal.AllocHGlobal(size);
            try
            {
                Marshal.StructureToPtr(structure, bufferIntPtr, true);
                Marshal.Copy(bufferIntPtr, buffer, 0, size);
            }
            finally
            {
                Marshal.FreeHGlobal(bufferIntPtr);
            }
            return buffer;
        }
    }
}
